#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <phidget22.h>
#include <math.h>
#include <time.h>

int attachcount = 0;
int thermocoupleSample = 0;
int ambientSample = 0;
int lightSample = 0;


//obtains current time for timestamping samples
char *time_stamp() {
	char *timestamp = (char *)malloc(sizeof(char) * 16);
	time_t ltime;
	ltime = time(NULL);
	struct tm *tm;
	tm = localtime(&ltime);
	sprintf(timestamp, "%02d:%02d:%02d", tm->tm_hour, tm->tm_min, tm->tm_sec);
	return timestamp;
}

static void CCONV onTemperatureAttachHandler(PhidgetHandle ph, void *ctx) { //runs when a temperature device attaches
	//since this handler is covering several devices we should start by checking what device this instance of the event was fired by
	int port;
	Phidget_getHubPort(ph, &port);
	switch (port)
	{ //do some initialization based on what device is attached
		case 0: //this was our thermocouple (see line 82)
			PhidgetTemperatureSensor_setDataInterval((PhidgetTemperatureSensorHandle)ph, 100); //10 samples/s
			PhidgetTemperatureSensor_setTemperatureChangeTrigger((PhidgetTemperatureSensorHandle)ph, 0); //report every sample regardless of if there's a change or not
			PhidgetTemperatureSensor_setThermocoupleType((PhidgetTemperatureSensorHandle)ph, THERMOCOUPLE_TYPE_K);
			break;
		case 1: //this was our ambient sensor (see line 83)
			PhidgetTemperatureSensor_setDataInterval((PhidgetTemperatureSensorHandle)ph, 500); //2 samples/s
			PhidgetTemperatureSensor_setTemperatureChangeTrigger((PhidgetTemperatureSensorHandle)ph, 0); //report every sample regardless of if there's a change or not
			break;
	}
	attachcount++;
}

static void CCONV onVoltageAttachHandler(PhidgetHandle ph, void *ctx) { //runs when a voltage input device attaches
	PhidgetVoltageInput_setDataInterval((PhidgetVoltageInputHandle)ph, 100); //10 samples/s
	PhidgetVoltageInput_setVoltageChangeTrigger((PhidgetVoltageInputHandle)ph, 0); //report every sample regardless of if there's a change or not

	attachcount++;
}

static void CCONV onDetachHandler(PhidgetHandle ph, void *ctx) { //runs when a device detaches
	attachcount--;
}

static void CCONV onErrorHandler(PhidgetHandle ph, void *ctx, Phidget_ErrorEventCode errorCode, const char *errorString) { //runs when an error occurs

	fprintf(stderr, "[Phidget Error Event] -> %s (%d)\n", errorString, errorCode);
}

static void CCONV onTemperatureChangeHandler(PhidgetTemperatureSensorHandle ph, void *ctx, double temperature) { //runs when a temp device reports a temp change
	FILE *file = (FILE *)ctx;
	if (attachcount != 3) //let's only start logging once all 3 devices are attached (for no particular reason, just because)
		return;
	printf("."); //provide some feedback that samples are being recorded
	fflush(stdout);
	//check which of our 2 temperature sensors has had a data event
	int port;
	Phidget_getHubPort((PhidgetHandle)ph, &port);
	if (port == 0) //thermocouple
	{
		fprintf(file, "Thermocouple,%f,%d,%s\n", temperature, thermocoupleSample++, time_stamp());
	}
	if (port == 1) //ambient
	{
		fprintf(file, "Ambient,%f,%d,%s\n", temperature, ambientSample++,time_stamp());
	}
}

static void CCONV onVoltageChangeHandler(PhidgetVoltageInputHandle ph, void *ctx, double voltage){ //runs when analog sensor reports voltage change
	FILE *file = (FILE *)ctx;
	if (attachcount != 3) //let's only start logging once all 3 devices are attached (for no particular reason, just because)
		return;
	printf("."); //provide some feedback that samples are being recorded
	fflush(stdout);
	//we are only using a single voltage input so we won't need to check which device fired this event, it can only be the one
	double luminosity = exp(0.02448*(voltage*200) - 0.8263); //calculate lux value from voltage output according to equation from 1143 user guide
	fprintf(file, "Light,%f,%d,%s\n", luminosity, lightSample++, time_stamp());
}

int main() {
	//our three devices
	PhidgetTemperatureSensorHandle thermocouple = NULL; //this will be our thermocouple
	PhidgetTemperatureSensorHandle ambient = NULL; //this will be our ambient sensor
	PhidgetVoltageInputHandle analogLight = NULL; //this will be the 1143

	//file we want to log to
	FILE *file = fopen("phidgetLog.csv", "w");

	fprintf(file, "Device,Sensor Value,Sample Number,Time Stamp\n");
	
	//allocate memory space for them
	PhidgetTemperatureSensor_create(&thermocouple);
	PhidgetTemperatureSensor_create(&ambient);
	PhidgetVoltageInput_create(&analogLight);

	//register handlers for each device
	Phidget_setOnAttachHandler((PhidgetHandle)thermocouple, onTemperatureAttachHandler, NULL);
	Phidget_setOnDetachHandler((PhidgetHandle)thermocouple, onDetachHandler, NULL);
	Phidget_setOnErrorHandler((PhidgetHandle)thermocouple, onErrorHandler, NULL);
	PhidgetTemperatureSensor_setOnTemperatureChangeHandler(thermocouple, onTemperatureChangeHandler, (void *)file);

	Phidget_setOnAttachHandler((PhidgetHandle)ambient, onTemperatureAttachHandler, NULL);
	Phidget_setOnDetachHandler((PhidgetHandle)ambient, onDetachHandler, NULL);
	Phidget_setOnErrorHandler((PhidgetHandle)ambient, onErrorHandler, NULL);
	PhidgetTemperatureSensor_setOnTemperatureChangeHandler(ambient, onTemperatureChangeHandler, (void *)file);

	Phidget_setOnAttachHandler((PhidgetHandle)analogLight, onVoltageAttachHandler, NULL);
	Phidget_setOnDetachHandler((PhidgetHandle)analogLight, onDetachHandler, NULL);
	Phidget_setOnErrorHandler((PhidgetHandle)analogLight, onErrorHandler, NULL);
	PhidgetVoltageInput_setOnVoltageChangeHandler(analogLight, onVoltageChangeHandler, (void *)file);

	//set parameters for each device so the open call knows where to look
	//****note that the serial number used here is for the VINT hub used in testing.  yours will be different so you should change this.
	Phidget_setDeviceSerialNumber((PhidgetHandle)thermocouple, 371089);
	Phidget_setDeviceSerialNumber((PhidgetHandle)ambient, 371089);
	Phidget_setDeviceSerialNumber((PhidgetHandle)analogLight, 371089);

	Phidget_setHubPort((PhidgetHandle)thermocouple, 0);
	Phidget_setHubPort((PhidgetHandle)ambient, 1);
	Phidget_setHubPort((PhidgetHandle)analogLight, 2);

	Phidget_setIsHubPortDevice((PhidgetHandle)analogLight, 1);

	//open each device
	Phidget_openWaitForAttachment((PhidgetHandle)thermocouple, 5000);
	Phidget_openWaitForAttachment((PhidgetHandle)ambient, 5000);
	Phidget_openWaitForAttachment((PhidgetHandle)analogLight, 5000);

	//wait for data to come in or user to terminate
	printf("Press ENTER to end program.\n");
	getchar();

	//unregister handlers
	PhidgetTemperatureSensor_setOnTemperatureChangeHandler(thermocouple, NULL, NULL);
	PhidgetTemperatureSensor_setOnTemperatureChangeHandler(ambient, NULL, NULL);
	PhidgetVoltageInput_setOnVoltageChangeHandler(analogLight, NULL, NULL);
	
	//deallocate memory
	Phidget_close((PhidgetHandle)thermocouple);
	Phidget_close((PhidgetHandle)ambient);
	Phidget_close((PhidgetHandle)analogLight);
	PhidgetTemperatureSensor_delete(&thermocouple);
	PhidgetTemperatureSensor_delete(&ambient);
	PhidgetVoltageInput_delete(&analogLight);

	fclose(file);

	//terminate
	return 0;

}

